@@ -11,22 +11,24 @@ module Yesod.Persist
 
 import Database.Persist
 import Database.Persist.TH
-import Control.Monad.Trans.Class (MonadTrans (..))
-import Control.Monad.IO.Class (MonadIO)
-import Control.Failure (Failure)
+import Control.Monad.Trans.Class (MonadTrans)
 
 import Yesod.Handler
 
-type YesodDB sub master = YesodPersistBackend master (GGHandler sub master IO)
+type YesodDB sub master = YesodPersistBackend master (GHandler sub master)
 
 class YesodPersist master where
     type YesodPersistBackend master :: (* -> *) -> * -> *
-    runDB :: MonadIO monad => YesodDB sub master a -> GGHandler sub master monad a
+    runDB :: YesodDB sub master a -> GHandler sub master a
 
 -- | Get the given entity by ID, or return a 404 not found if it doesn't exist.
-get404 :: (PersistBackend t m, PersistEntity val, Monad (t m),
-           Failure ErrorResponse m, MonadTrans t)
-       => Key t val -> t m val
+get404 :: ( PersistStore b m
+          , PersistEntity val
+          , Monad (b m)
+          , m ~ GHandler sub master
+          , MonadTrans b
+          )
+       => Key b val -> b m val
 get404 key = do
     mres <- get key
     case mres of
@@ -35,9 +37,14 @@ get404 key = do
 
 -- | Get the given entity by unique key, or return a 404 not found if it doesn't
 --   exist.
-getBy404 :: (PersistBackend t m, PersistEntity val, Monad (t m),
-             Failure ErrorResponse m, MonadTrans t)
-         => Unique val t -> t m (Key t val, val)
+getBy404 :: ( PersistUnique b m
+            , PersistEntity val
+            , m ~ GHandler sub master
+            , Monad (b m)
+            , MonadTrans b
+            , PersistEntityBackend val ~ b
+            )
+         => Unique val b -> b m (Entity val)
 getBy404 key = do
     mres <- getBy key
     case mres of
